package nl.sbdeveloper.themeparkplus;

import net.milkbowl.vault.economy.Economy;
import nl.sbdeveloper.themeparkplus.commands.TPPCMD;
import nl.sbdeveloper.themeparkplus.commands.TPPTabComplete;
import nl.sbdeveloper.themeparkplus.listeners.*;
import nl.sbdeveloper.themeparkplus.managers.DBManager;
import nl.sbdeveloper.themeparkplus.sbutils.UpdateManager;
import nl.sbdeveloper.themeparkplus.sbutils.YamlFile;
import nl.sbdeveloper.themeparkplus.util.License;
import org.bstats.bukkit.Metrics;
import org.bukkit.Bukkit;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.RegisteredServiceProvider;
import org.bukkit.plugin.java.JavaPlugin;
import org.jetbrains.annotations.NotNull;

import java.io.InputStreamReader;
import java.io.Reader;
import java.nio.charset.StandardCharsets;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Objects;

public final class ThemeParkPlus extends JavaPlugin {

    private static ThemeParkPlus instance;
    private static YamlFile config;
    private static DBManager data;
    private static YamlFile messages;

    private static Economy econ = null;
    private static String webhookURL;

    private final int configVersion = 2;

    @Override
    public void onEnable() {
        instance = this;

        Bukkit.getLogger().info("[ThemeParkPlus] -------------------------------");
        Bukkit.getLogger().info("[ThemeParkPlus] ThemeParkPlus v" + this.getDescription().getVersion());
        Bukkit.getLogger().info("[ThemeParkPlus] Made by SBDeveloper");

        Bukkit.getLogger().info("[ThemeParkPlus]  ");

        Bukkit.getLogger().info("[ThemeParkPlus] Loading Files...");

        config = new YamlFile("config");
        config.loadDefaults();

        data = new DBManager("data");

        messages = new YamlFile("messages");
        messages.loadDefaults();

        Reader defConfigStream = new InputStreamReader(Objects.requireNonNull(getResource("config.yml")), StandardCharsets.UTF_8);
        YamlConfiguration defConfig = YamlConfiguration.loadConfiguration(defConfigStream);
        config.getFile().setDefaults(defConfig);
        config.getFile().options().copyDefaults(true);
        config.saveFile();

        if (!config.getFile().contains("Version") || Objects.equals(config.getFile().getString("Version"), "")) {
            config.getFile().set("Version", configVersion);
            config.saveFile();
        }

        if (!Objects.equals(config.getFile().getString("Version"), String.valueOf(configVersion))) {
            Bukkit.getLogger().info("[ThemeParkPlus] Updating outdated config...");
            updateConfig();
        }

        Bukkit.getLogger().info("[ThemeParkPlus] Checking license...");

        if (config.getFile().contains("License")) {
            Bukkit.getLogger().info("[ThemeParkPlus] Licence code: " + config.getFile().getString("License"));
        } else {
            Bukkit.getLogger().severe("[ThemeParkPlus] Licence code unknown! Please change the config.yml!");
            return;
        }

        new License(this, "TP", config.getFile().getString("License"));

        if (getSConfig().getFile().getBoolean("UpdateChecker.Enabled")) {
            UpdateManager updateManager = new UpdateManager(this, 6, UpdateManager.CheckType.SBDPLUGINS);

            updateManager.handleResponse((versionResponse, version) -> {
                if (versionResponse == UpdateManager.VersionResponse.FOUND_NEW) {
                    Bukkit.getLogger().warning("[ThemeParkPlus] There is a new version available! Current: " + this.getDescription().getVersion() + " New: " + version.get());
                    if (getSConfig().getFile().getBoolean("UpdateChecker.DownloadOnUpdate")) {
                        Bukkit.getLogger().info("[ThemeParkPlus] Trying to download the update...");

                        updateManager.handleDownloadResponse((downloadResponse, fileName) -> {
                            if (downloadResponse == UpdateManager.DownloadResponse.DONE) {
                                Bukkit.getLogger().info("[ThemeParkPlus] Update downloaded! If you restart your server, it will be loaded. Filename: " + fileName);
                            } else if (downloadResponse == UpdateManager.DownloadResponse.ERROR) {
                                Bukkit.getLogger().severe("[ThemeParkPlus] Something went wrong when trying downloading the latest version.");
                            } else if (downloadResponse == UpdateManager.DownloadResponse.UNAVAILABLE) {
                                Bukkit.getLogger().warning("[ThemeParkPlus] Unable to download the latest version.");
                            }
                        }).runUpdate();
                    }
                } else if (versionResponse == UpdateManager.VersionResponse.LATEST) {
                    Bukkit.getLogger().info("[ThemeParkPlus] You are running the latest version [" + this.getDescription().getVersion() + "]!");
                } else if (versionResponse == UpdateManager.VersionResponse.UNAVAILABLE) {
                    Bukkit.getLogger().severe("[ThemeParkPlus] Unable to perform an update check.");
                }
            }).check();
        }

        if (Bukkit.getPluginManager().getPlugin("ThemePark") == null) {
            Bukkit.getLogger().severe("[ThemeParkPlus] Missing ThemePark! Please install it first.");
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        if (Bukkit.getPluginManager().getPlugin("WorldEdit") == null) {
            Bukkit.getLogger().severe("[ThemeParkPlus] Missing WorldEdit! Please install it first.");
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        if (Bukkit.getPluginManager().getPlugin("WorldGuard") == null) {
            Bukkit.getLogger().severe("[ThemeParkPlus] Missing WorldGuard! Please install it first.");
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        if (!setupEconomy()) {
            Bukkit.getLogger().severe("[ThemeParkPlus] Missing Vault! Please install it first.");
            getServer().getPluginManager().disablePlugin(this);
            return;
        }

        Bukkit.getLogger().info("[ThemeParkPlus] Loading commands...");
        getCommand("themeparkplus").setExecutor(new TPPCMD());
        getCommand("themeparkplus").setTabCompleter(new TPPTabComplete());

        Bukkit.getLogger().info("[ThemeParkPlus] Loading listeners...");
        Bukkit.getPluginManager().registerEvents(new DirectionalGateListener(), this);
        Bukkit.getPluginManager().registerEvents(new FastpassListeners(), this);
        Bukkit.getPluginManager().registerEvents(new SignListeners(), this);
        Bukkit.getPluginManager().registerEvents(new WaitingTimeListener(), this);
        if (getSConfig().getFile().getBoolean("AntiFreerun.Enabled")) {
            Bukkit.getPluginManager().registerEvents(new AntiFreerunListener(), this);
        }

        if (getSConfig().getFile().getBoolean("DiscordWebhook.Enabled")) {
            String URL = getSConfig().getFile().getString("DiscordWebhook.WebhookURL");
            if (URL != null) {
                Bukkit.getPluginManager().registerEvents(new StatusChangeListener(), this);

                Bukkit.getLogger().info("[ThemeParkPlus] Loading Discord webhook...");
                webhookURL = URL;
            } else {
                Bukkit.getLogger().severe("[ThemeParkPlus] Couldn't load the webhook builder! The URL is null.");
            }
        }

        Bukkit.getLogger().info("[ThemeParkPlus] Loading data...");
        try {
            data.load();
        } catch (SQLException e) {
            e.printStackTrace();
            Bukkit.getLogger().severe("[ThemeParkPlus] Couldn't load data! Something went wrong.");
        }

        Bukkit.getLogger().info("[ThemeParkPlus] Loading metrics (can be disabled in the config of bStats)...");
        new Metrics(this, 5023);

        Bukkit.getLogger().info("[ThemeParkPlus] Plugin enabled!");
        Bukkit.getLogger().info("[ThemeParkPlus] -------------------------------");
    }

    @Override
    public void onDisable() {
        Bukkit.getLogger().info("[ThemeParkPlus] Saving data to data file...");
        data.save();

        Bukkit.getLogger().info("[ThemeParkPlus] Plugin disabled!");
        instance = null;
    }

    public static ThemeParkPlus getInstance() {
        return instance;
    }

    public static YamlFile getSConfig() {
        return config;
    }

    public static DBManager getData() {
        return data;
    }

    public static YamlFile getMessages() {
        return messages;
    }

    public static Economy getEconomy() {
        return econ;
    }

    public static String getWebhookURL() {
        return webhookURL;
    }

    private boolean setupEconomy() {
        if (getServer().getPluginManager().getPlugin("Vault") == null) {
            return false;
        }
        RegisteredServiceProvider<Economy> rsp = getServer().getServicesManager().getRegistration(Economy.class);
        if (rsp == null) {
            return false;
        }
        econ = rsp.getProvider();
        return econ != null;
    }

    private void updateConfig() {
        HashMap< String, Object > newConfig = getConfigVals();
        for (String var: config.getFile().getKeys(false)) {
            newConfig.remove(var);
        }
        if (newConfig.size() != 0) {
            for (String key: newConfig.keySet()) {
                config.getFile().set(key, newConfig.get(key));
            }
            config.getFile().set("Version", configVersion);
            config.saveFile();
        }
    }

    @NotNull
    private HashMap<String, Object> getConfigVals() {
        HashMap<String, Object> var = new HashMap <>();
        Reader defConfigStream = new InputStreamReader(Objects.requireNonNull(getResource("config.yml")), StandardCharsets.UTF_8);
        YamlConfiguration defConfig = YamlConfiguration.loadConfiguration(defConfigStream);
        for (String key : defConfig.getKeys(false)) {
            var.put(key, defConfig.get(key));
        }
        return var;
    }
}
